#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Main code implemented for the Master's Thesis research,
Master of Science in Business, Major in Finance.
Authors: Lars Fredrik Osen and Jacob-Sterling Deehr Johannessen

"""

import pandas as pd
import numpy as np
from pandas.tseries.offsets import *
from scipy import stats
import datetime as dt
from pandas.tseries.offsets import MonthBegin
import matplotlib.pyplot as plt

# LOAD ALL DATA 
SPX = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/SPX.xlsx')
SPX_index = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/SPX_index.xlsx', index_col=0)
M2 = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/M2.xlsx', index_col = 0)
GOLD = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/GOLD.xlsx', index_col = 0)
RF = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/RF.xlsx', index_col = 0)
CPI = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/CPIAUCNS.xlsx', index_col = 0)
WTI = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/WTI.xlsx', index_col = 0)


#### Calculating returns #### 

# GOLD Monthly Returns
GOLD_M_ret = GOLD.pct_change().resample('M').agg(lambda x: (x + 1).prod() - 1)

# CPIAUCSL % Change
CPI_M_ret = CPI.pct_change().resample('M').agg(lambda x: (x + 1).prod() - 1)

# M2 % Change
M2_M_ret = M2.pct_change().resample('M').agg(lambda x: (x + 1).prod() - 1)

# SPX Index Change
SPX_index_ret = SPX_index.pct_change().resample('M').agg(lambda x: (x + 1).prod() - 1)




SPX['date'] = pd.to_datetime(SPX['Date'])
SPX.set_index('date', inplace=True)

SPX_M = SPX.resample('M').last()

stock_prices_melted = pd.melt(SPX_M, id_vars=['Date'], var_name='stock', value_name='price')

stock_prices_melted['Date'] = pd.to_datetime(stock_prices_melted['Date'])


# Create a dictionary mapping stocks to ID_nos
stock_to_ID_no = dict(zip(stock_prices_melted['stock'].unique(), range(1, len(stock_prices_melted['stock'].unique())+1)))

# Map stocks to ID_nos and add as a new column to the DataFrame
stock_prices_melted['ID_no'] = pd.factorize(stock_prices_melted['stock'], sort=True)[0] + 1


# Sort DataFrame by date and ID_no
stock_prices_melted = stock_prices_melted.sort_values(['Date', 'ID_no'])

# Group DataFrame by ID_no and calculate monthly returns
stock_prices_melted['ret'] = stock_prices_melted.groupby('ID_no')['price'].pct_change()


## Rename stock prices_melted to SPX_LP_M
SPX_LP_m = stock_prices_melted

SPX_LP_m.tail()

# Line up date to be end of month
SPX_LP_m['Date']=pd.to_datetime(SPX_LP_m['Date'])



#######################################################
# Create Momentum Portfolio                           #   
# Measures Based on Past (J) Month Compounded Returns #
#######################################################

# Define strategy

J = 6
K = 6

# Calculate rolling cumulative return
# by summing log(1+ret) over the formation period
_tmp_SPX_LP = SPX_LP_m[['ID_no', 'Date', 'ret']].sort_values(['ID_no', 'Date']).set_index('Date')

# Replace missing return with 0
_tmp_SPX_LP['ret'] = _tmp_SPX_LP['ret'].fillna(0)

# Calculate rolling cumulative return
# by summing log(1+ret) over the formation period
_tmp_SPX_LP['logret'] = np.log(1 + _tmp_SPX_LP['ret'])
umd = _tmp_SPX_LP.groupby(['ID_no'])['logret'].rolling(J, min_periods=J).sum().reset_index()

# Calculate 'cumret' from 'logret'
umd['cumret'] = np.exp(umd['logret']) - 1

# Replace NaN values with 0
umd[['cumret', 'logret']] = umd[['cumret', 'logret']].fillna(0)

# Shift cumret and logret up by K rows to remove the first K NaN values
umd[['cumret', 'logret']] = umd.groupby('ID_no')[['cumret', 'logret']].shift(K)

# Drop the first K rows from the DataFrame
umd = umd.dropna()


########################################
# Formation of 10 Momentum Portfolios  #
########################################

# For each date: assign ranking 1-10 based on cumret
# 1=lowest 10=highest cumret
umd=umd.dropna(axis=0, subset=['cumret'])

umd['momr'] = umd.groupby('Date')['cumret'].transform(lambda x: pd.qcut(x[x != 0], 10, labels=False, duplicates='drop') if not x[x != 0].empty else pd.Series(np.nan, index=x.index))

# Calculate hdate1 and hdate2 using medate
umd['form_date'] = umd['Date']
umd['medate'] = umd['Date']+MonthEnd(0)
umd['hdate1']=umd['medate']+MonthBegin(1)
umd['hdate2']=umd['medate']+MonthEnd(K)
umd = umd[['ID_no', 'form_date','momr','hdate1','hdate2']]

# join rank and return data together
# note to self: this step consumes a lot of memory so takes a while
_tmp_ret = SPX_LP_m[['ID_no','Date','ret']]
port = pd.merge(_tmp_ret, umd, on=['ID_no'], how='inner')
port = port[(port['hdate1']<=port['Date']) & (port['Date']<=port['hdate2'])]

umd2 = port.sort_values(by=['Date','momr','form_date','ID_no']).drop_duplicates()
umd3 = umd2.groupby(['Date','momr','form_date'])['ret'].mean().reset_index()

# Skip first two years of the sample 
start_yr = umd3['Date'].dt.year.min()+2
umd3 = umd3[umd3['Date'].dt.year>=start_yr]
umd3 = umd3.sort_values(by=['Date','momr'])

# Create one return series per MOM group every month
ewret = umd3.groupby(['Date','momr'])['ret'].mean().reset_index()
ewstd = umd3.groupby(['Date','momr'])['ret'].std().reset_index()
ewret = ewret.rename(columns={'ret':'ewret'})
ewstd = ewstd.rename(columns={'ret':'ewretstd'})
ewretdat = pd.merge(ewret, ewstd, on=['Date','momr'], how='inner')
ewretdat = ewretdat.sort_values(by=['momr'])

# Portfolio summary
ewretdat.groupby(['momr'])['ewret'].describe()[['count','mean', 'std']]

#################################
# Long-Short Portfolio Returns  #
#################################

# Transpose portfolio layout to have columns as portfolio returns
ewretdat2 = ewretdat.pivot(index='Date', columns='momr', values='ewret')

# Add prefix port in front of each column
ewretdat2 = ewretdat2.add_prefix('port')
ewretdat2 = ewretdat2.rename(columns={'port0.0':'losers', 'port9.0':'winners'})
ewretdat2['long_short'] = ewretdat2['winners'] - ewretdat2['losers']

# Compute Long-Short Portfolio Cumulative Returns
ewretdat3 = ewretdat2
ewretdat3['1+losers']=1+ewretdat3['losers']
ewretdat3['1+winners']=1+ewretdat3['winners']
ewretdat3['1+ls'] = 1+ewretdat3['long_short']

ewretdat3['cumret_winners']=ewretdat3['1+winners'].cumprod()-1
ewretdat3['cumret_losers']=ewretdat3['1+losers'].cumprod()-1
ewretdat3['cumret_long_short']=ewretdat3['1+ls'].cumprod()-1

#################################
# Portfolio Summary Statistics  #
################################# 

# Mean 
mom_mean = ewretdat3[['winners', 'losers', 'long_short']].mean().to_frame()
mom_mean = mom_mean.rename(columns={0:'mean'}).reset_index()

# T-Value and P-Value
t_losers = pd.Series(stats.ttest_1samp(ewretdat3['losers'],0.0)).to_frame().T
t_winners = pd.Series(stats.ttest_1samp(ewretdat3['winners'],0.0)).to_frame().T
t_long_short = pd.Series(stats.ttest_1samp(ewretdat3['long_short'],0.0)).to_frame().T

t_losers['momr']='losers'
t_winners['momr']='winners'
t_long_short['momr']='long_short'

t_output =pd.concat([t_winners, t_losers, t_long_short])\
    .rename(columns={0:'t-stat', 1:'p-value'})

# Combine mean, t and p
mom_output = pd.merge(mom_mean, t_output, on=['momr'], how='inner')

## Data import
import pandas as pd
import statsmodels.api as sm
from statsmodels.tsa.api import VAR

### Methodology 
W = ewretdat3['winners'].resample('M').last()
L = ewretdat3['losers'].resample('M').last()
LS = ewretdat3['long_short'].resample('M').last()

momentum_portfolio = LS

M2_last = M2['M2'].resample('M').last()
CPI_last = CPI['CPIAUCSL'].resample('M').last()
GOLD_last = GOLD['Last Price'].resample('M').last()
WTI_last = WTI['Price'].resample('M').last()
M2_last = M2['M2'].resample('M').last()
RF_last = RF["RF"].resample('M').last()

### Create data frame with price data

df_price = pd.DataFrame({
    'longshort' : momentum_portfolio,
    'rf': RF_last,
    'm2': M2_last,
    'cpi': CPI_last,
    'gold': GOLD_last,
    'WTI': WTI_last
})

df_price = df_price.dropna()

df_before_nov_2008 = df_price[df_price.index < '2008-11-01']
df_after_nov_2008 = df_price[df_price.index >= '2008-11-01']

# merge the dataframes
df_price = pd.concat([momentum_portfolio, M2_last, CPI_last, GOLD_last, WTI_last,RF_last], axis=1)

# rename the columns
df_price.columns = ['longshort', 'm2', 'cpi', 'gold', 'WTI','RF']
df_price = df_price.dropna()

# check the data
print(df_price.head())


import matplotlib.pyplot as plt

df_price.plot(subplots=True, figsize=(10, 15))
plt.tight_layout()
plt.show()


from statsmodels.tsa.vector_ar.vecm import coint_johansen


#####################################################################################
######### When testing for co-integration we are testing all the variables together##
#####################################################################################
def cointegration_test(df):
    result = coint_johansen(df, 0, 1)
    d = {'0.90':0, '0.95':1, '0.99':2}
    traces = result.lr1
    cvts = result.cvt[:, d['0.95']]  # 0: 90%  1:95% 2: 99%
    def adjust(val, length= 6): return str(val).ljust(length)
    
    # Summary
    print('Name   ::  Test Stat > C(95%)    =>   Signif \n', '--'*20)
    for col, trace, cvt in zip(df.columns, traces, cvts):
        print(adjust(col), ':: ', adjust(round(trace,4), 9), ">", adjust(cvt, 8), ' =>  ' , trace > cvt)

cointegration_test(df_before_nov_2008)
cointegration_test(df_after_nov_2008)


#######################################
##### Augmented Dickey-Fuller test#####
#######################################

from statsmodels.tsa.stattools import adfuller

    
for column in df_price.columns:
    result = adfuller(df_price[column], autolag='AIC')
    # Extracting the values from the results:
    print(f'ADF Statistic for {column}: {result[0]}')
    print(f'p-value: {result[1]}')
    print(f'Number of Lags Used: {result[2]}')
    print(f'Number of Observations Used: {result[3]}')   
    print('Critical Values:')
    for key, value in result[4].items():
        print(f'\t{key}: {value:.3f}')
    
    if result[0] < result[4]["5%"]:
        print ("Reject Ho - Time Series for", column, "is Stationary")
    else:
        print ("Failed to Reject Ho - Time Series for", column, "is Non-Stationary")
    
    print('\n')
    
    
for column in df_before_nov_2008.columns:
    result = adfuller(df_before_nov_2008[column], autolag='AIC')
    # Extracting the values from the results:
    print(f'ADF Statistic for {column}: {result[0]}')
    print(f'p-value: {result[1]}')
    print(f'Number of Lags Used: {result[2]}')
    print(f'Number of Observations Used: {result[3]}')
    print('Critical Values:')
    for key, value in result[4].items():
        print(f'\t{key}: {value:.3f}')
    
    if result[0] < result[4]["5%"]:
        print ("Reject Ho - Time Series for", column, "is Stationary")
    else:
        print ("Failed to Reject Ho - Time Series for", column, "is Non-Stationary")
    
    print('\n')

for column in df_after_nov_2008.columns:
    result = adfuller(df_after_nov_2008[column], autolag='AIC')
    # Extracting the values from the results:
    print(f'ADF Statistic for {column}: {result[0]}')
    print(f'p-value: {result[1]}')
    print(f'Number of Lags Used: {result[2]}')
    print(f'Number of Observations Used: {result[3]}')
    print('Critical Values:')
    for key, value in result[4].items():
        print(f'\t{key}: {value:.3f}')
    
    if result[0] < result[4]["5%"]:
        print ("Reject Ho - Time Series for", column, "is Stationary")
    else:
        print ("Failed to Reject Ho - Time Series for", column, "is Non-Stationary")
    
    print('\n')


### Differencing ADF for stationarity

import numpy as np

M2_log_returns = np.log(M2_last / M2_last.shift(1)).dropna()
CPI_log_returns = np.log(CPI_last / CPI_last.shift(1)).dropna()
GOLD_log_returns = np.log(GOLD_last / GOLD_last.shift(1)).dropna()
WTI_log_returns = np.log(WTI_last / WTI_last.shift(1)).dropna()
RF_log_returns = np.log(RF_last / RF_last.shift(1)).dropna()

# New data frame for ADF test

# Total period
df_diff = pd.DataFrame({
    'longshort' : momentum_portfolio,
    'rf': RF_log_returns,
    'm2': M2_log_returns,
    'cpi': CPI_log_returns,
    'gold': GOLD_log_returns,
    'WTI': WTI_log_returns
})

df_diff = df_diff.dropna()

df_before_nov_2008_diff = df_diff[df_diff.index < '2008-11-01']
df_after_nov_2008_diff = df_diff[df_diff.index >= '2008-11-01']


import matplotlib.pyplot as plt
df_before_nov_2008_diff.plot(subplots=True, figsize=(10, 15))
plt.tight_layout()
plt.show()

df_after_nov_2008_diff.plot(subplots=True, figsize=(10, 15))
plt.tight_layout()
plt.show()

import matplotlib.pyplot as plt

# Plot log returns for RF
plt.figure(figsize=(12, 6))
plt.plot(RF_log_returns.index, RF_log_returns, label='RF log returns')
plt.axhline(y=RF_log_returns.mean(), color='r', linestyle='-', label='Mean')

plt.title('RF Log Returns')
plt.xlabel('Date')
plt.ylabel('Log Returns')
plt.legend()
plt.show()

### Running ADF again for stationarity

from statsmodels.tsa.stattools import adfuller

# ADF Test for each series in df
for column in df_diff.columns:
    result = adfuller(df_diff[column], autolag='AIC')
    # Extracting the values from the results:
    print(f'ADF Statistic for {column}: {result[0]}')
    print(f'p-value: {result[1]}')
    print(f'Number of Lags Used: {result[2]}')
    print(f'Number of Observations Used: {result[3]}')   
    print('Critical Values:')
    for key, value in result[4].items():
        print(f'\t{key}: {value:.3f}')
    
    if result[0] < result[4]["5%"]:
        print ("Reject Ho - Time Series for", column, "is Stationary")
    else:
        print ("Failed to Reject Ho - Time Series for", column, "is Non-Stationary")
    
    print('\n')  # print a newline for better readability

for column in df_before_nov_2008_diff.columns:
    result = adfuller(df_before_nov_2008_diff[column], autolag='AIC')
    # Extracting the values from the results:
    print(f'ADF Statistic for {column}: {result[0]}')
    print(f'p-value: {result[1]}')
    print(f'Number of Lags Used: {result[2]}')
    print(f'Number of Observations Used: {result[3]}')
    print('Critical Values:')
    for key, value in result[4].items():
        print(f'\t{key}: {value:.3f}')
    
    if result[0] < result[4]["5%"]:
        print ("Reject Ho - Time Series for", column, "is Stationary")
    else:
        print ("Failed to Reject Ho - Time Series for", column, "is Non-Stationary")
    
    print('\n')



for column in df_after_nov_2008_diff.columns:
    result = adfuller(df_after_nov_2008_diff[column], autolag='AIC')
    # Extracting the values from the results:
    print(f'ADF Statistic for {column}: {result[0]}')
    print(f'p-value: {result[1]}')
    print(f'Number of Lags Used: {result[2]}')
    print(f'Number of Observations Used: {result[3]}')
    print('Critical Values:')
    for key, value in result[4].items():
        print(f'\t{key}: {value:.3f}')
    
    if result[0] < result[4]["5%"]:
        print ("Reject Ho - Time Series for", column, "is Stationary")
    else:
        print ("Failed to Reject Ho - Time Series for", column, "is Non-Stationary")
    
    print('\n')


### KPSS Test
import pandas as pd
from statsmodels.tsa.stattools import kpss

variables = ['longshort', 'rf', 'cpi', 'm2','gold','WTI']

dataframes = {'df_after_nov_2008': df_after_nov_2008_diff,
              'df_before_nov_2008': df_before_nov_2008_diff}

for df_name, df in dataframes.items():
    print(f'\nKPSS Test Results for {df_name}:\n')
    for var in variables:
        statistic, p_value, n_lags, critical_values = kpss(df[var],nlags='auto')
        print(f'\nResults for {var}:\n')
        print(f'KPSS Statistic: {statistic}')
        print(f'p-value: {p_value}')
        print(f'num lags: {n_lags}')
        print('Critical Values:')
        for key, value in critical_values.items():
            print(f'   {key} : {value}')
            print(f'KPSS Statistic: {statistic}')
        if statistic < critical_values['5%']:
            print('The series is stationary at 5% significance level')
        else:
            print('The series is not stationary at 5% significance level')


### Differencing ADF for stationarity
import numpy as np

M2_log_returns_diff = M2_log_returns.diff().dropna()
CPI_log_returns_diff = CPI_log_returns.diff().dropna()
GOLD_log_returns_diff = GOLD_log_returns.diff().dropna()
WTI_log_returns_diff = WTI_log_returns.diff().dropna()
RF_log_returns_diff = RF_log_returns.diff().dropna()
momentum_portfolio_diff = momentum_portfolio.diff().dropna()

# New data frame for ADF test

# Total period
df_diff_diff = pd.DataFrame({
    'longshort' : momentum_portfolio_diff,
    'rf': RF_log_returns_diff,
    'm2': M2_log_returns_diff,
    'cpi': CPI_log_returns_diff,
    'gold': GOLD_log_returns_diff,
    'WTI': WTI_log_returns_diff
})

df_diff_diff = df_diff_diff.dropna()

df_before_nov_2008_diff_diff = df_diff_diff[df_diff_diff.index < '2008-11-01']
df_after_nov_2008_diff_diff = df_diff_diff[df_diff_diff.index >= '2008-11-01']

# Export to Excel
df_before_nov_2008_diff_diff.to_excel('df_before_nov_20086x12.xlsx')
df_after_nov_2008_diff_diff.to_excel('df_after_nov_20086x12.xlsx')



### Plotting
import matplotlib.pyplot as plt
df_before_nov_2008_diff_diff.plot(subplots=True, figsize=(10, 15))
plt.tight_layout()
plt.show()



df_after_nov_2008_diff_diff.plot(subplots=True, figsize=(10, 15))
plt.tight_layout()
plt.show()

### Running ADF again for stationarity

from statsmodels.tsa.stattools import adfuller

# ADF Test for each series in df
for column in df_diff_diff.columns:
    result = adfuller(df_diff_diff[column], autolag='AIC',regression='ct')
    # Extracting the values from the results:
    print(f'ADF Statistic for {column}: {result[0]}')
    print(f'p-value: {result[1]}')
    print(f'Number of Lags Used: {result[2]}')
    print(f'Number of Observations Used: {result[3]}')
    print('Critical Values:')
    for key, value in result[4].items():
        print(f'\t{key}: {value:.3f}')
    
    if result[0] < result[4]["5%"]:
        print ("Reject Ho - Time Series for", column, "is Stationary")
    else:
        print ("Failed to Reject Ho - Time Series for", column, "is Non-Stationary")
    
    print('\n')

for column in df_before_nov_2008_diff_diff.columns:
    result = adfuller(df_before_nov_2008_diff_diff[column], autolag='AIC',regression="ct")
    # Extracting the values from the results:
    print(f'ADF Statistic for {column}: {result[0]}')
    print(f'p-value: {result[1]}')
    print(f'Number of Lags Used: {result[2]}')
    print(f'Number of Observations Used: {result[3]}')
    print('Critical Values:')
    for key, value in result[4].items():
        print(f'\t{key}: {value:.3f}')
    
    if result[0] < result[4]["5%"]:
        print ("Reject Ho - Time Series for", column, "is Stationary")
    else:
        print ("Failed to Reject Ho - Time Series for", column, "is Non-Stationary")
    
    print('\n')



for column in df_after_nov_2008_diff_diff.columns:
    result = adfuller(df_after_nov_2008_diff_diff[column], autolag='AIC',regression="ct")
    # Extracting the values from the results:
    print(f'ADF Statistic for {column}: {result[0]}')
    print(f'p-value: {result[1]}')
    print(f'Number of Lags Used: {result[2]}')
    print(f'Number of Observations Used: {result[3]}')
    print('Critical Values:')
    for key, value in result[4].items():
        print(f'\t{key}: {value:.3f}')
    
    if result[0] < result[4]["5%"]:
        print ("Reject Ho - Time Series for", column, "is Stationary")
    else:
        print ("Failed to Reject Ho - Time Series for", column, "is Non-Stationary")
    
    print('\n')


import pandas as pd
from statsmodels.tsa.stattools import kpss

# assuming df_after_nov_2008_without_m2 and df_before_nov_2008_without_m2 are your dataframes

variables = ['longshort', 'rf', 'cpi', 'm2','gold','WTI']

dataframes = {'df_after_nov_2008': df_after_nov_2008_diff_diff,
              'df_before_nov_2008': df_before_nov_2008_diff_diff}

for df_name, df in dataframes.items():
    print(f'\nKPSS Test Results for {df_name}:\n')
    for var in variables:
        statistic, p_value, n_lags, critical_values = kpss(df[var],nlags='legacy',regression="c")
        print(f'\nResults for {var}:\n')
        print(f'KPSS Statistic: {statistic}')
        print(f'p-value: {p_value}')
        print(f'num lags: {n_lags}')
        print('Critical Values:')
        for key, value in critical_values.items():
            print(f'   {key} : {value}')
            print(f'KPSS Statistic: {statistic}')
        if statistic < critical_values['5%']:
            print('The series is stationary at 5% significance level')
        else:
            print('The series is not stationary at 5% significance level')




import pandas as pd

# Read each file into a pandas DataFrame and set 'Date' as the index
df_after_nov_20089x1 = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/DF/df_after_nov_20089x1.xlsx', index_col='Date')
df_before_nov_20089x1 = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/DF/df_before_nov_20089x1.xlsx', index_col='Date')
df_after_nov_20086x12 = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/DF/df_after_nov_20086x12.xlsx', index_col='Date')
df_before_nov_20086x12 = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/DF/df_before_nov_20086x12.xlsx', index_col='Date')
df_after_nov_20086x6 = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/DF/df_after_nov_20086x6.xlsx', index_col='Date')
df_before_nov_20086x6 = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/DF/df_before_nov_20086x6.xlsx', index_col='Date')
df_after_nov_20086x3 = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/DF/df_after_nov_20086x3.xlsx', index_col='Date')
df_before_nov_20086x3 = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/DF/df_before_nov_20086x3.xlsx', index_col='Date')
df_after_nov_20081x1 = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/DF/df_after_nov_20081x1.xlsx', index_col='Date')
df_before_nov_20081x1 = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/DF/df_before_nov_20081x1.xlsx', index_col='Date')
df_after_nov_200812x12 = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/DF/df_after_nov_200812x12.xlsx', index_col='Date')
df_before_nov_200812x12 = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/DF/df_before_nov_200812x12.xlsx', index_col='Date')
df_after_nov_200812x1 = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/DF/df_after_nov_200812x1.xlsx', index_col='Date')
df_before_nov_200812x1 = pd.read_excel('/Users/larsfredrikosen/Desktop/MSC_DATA_THESIS/Data/DF/df_before_nov_200812x1.xlsx', index_col='Date')


import pandas as pd
import numpy as np
from statsmodels.tsa.stattools import grangercausalitytests
from statsmodels.regression.linear_model import OLS

# Store dataframes in a dictionary for easy access
dataframes = {
    'df_after_nov_20089x1': df_after_nov_20089x1,
    'df_before_nov_20089x1': df_before_nov_20089x1,
    'df_after_nov_20086x12': df_after_nov_20086x12,
    'df_before_nov_20086x12': df_before_nov_20086x12,
    'df_after_nov_20086x6': df_after_nov_20086x6,
    'df_before_nov_20086x6': df_before_nov_20086x6,
    'df_after_nov_20086x3': df_after_nov_20086x3,
    'df_before_nov_20086x3': df_before_nov_20086x3,
    'df_after_nov_20081x1': df_after_nov_20081x1,
    'df_before_nov_20081x1': df_before_nov_20081x1,
    'df_after_nov_200812x12': df_after_nov_200812x12,
    'df_before_nov_200812x12': df_before_nov_200812x12,
    'df_after_nov_200812x1': df_after_nov_200812x1,
    'df_before_nov_200812x1': df_before_nov_200812x1,
}


##############################
### Vector Autoregression ####
##############################

import pandas as pd

# Define strategy

ub = df_before_nov_20086x6
ua = df_after_nov_20086x6


# Run the VAR model for pre Nov 2008
model_before_nov_2008 = VAR(ub)
results_before_nov_2008 = model_before_nov_2008.fit(ic='aic',trend='c')

# Run the VAR model for post Nov 2008
model_after_nov_2008 = VAR(ua)
results_after_nov_2008 = model_after_nov_2008.fit(ic='aic',trend='c')

# Print summary of each model
print(results_before_nov_2008.summary())
print(results_after_nov_2008.summary())



from statsmodels.stats.stattools import durbin_watson
from statsmodels.stats.diagnostic import het_breuschpagan

######## Autocorrelation test
dw_result_before = durbin_watson(results_before_nov_2008.resid)
dw_result_after = durbin_watson(results_after_nov_2008.resid)
print('Durbin-Watson test statistic before Nov 2008:', dw_result_before)
print('Durbin-Watson test statistic after Nov 2008:', dw_result_after)

######## Heteroscedasticity test

residuals_before = results_before_nov_2008.resid
residuals_after = results_after_nov_2008.resid

# Assuming your dataframes df_before_nov_2008 and df_after_nov_2008 

from statsmodels.stats.diagnostic import het_white
from statsmodels.tools.tools import add_constant
# Define the list of variables
variables = results_before_nov_2008.names

# Run the White test for each variable
for var in variables:
    print(f"Running White's test for variable: {var}")
    
    # Select the residuals for the current variable
    residuals_before_var = residuals_before[var].values.reshape(-1)
    residuals_after_var = residuals_after[var].values.reshape(-1)

    # If the lengths of residuals and df are not the same, truncate the longer one
    if len(residuals_before_var) != len(ub):
        min_len = min(len(residuals_before_var), len(ub))
        residuals_before_var = residuals_before_var[:min_len]
        ub = ub.iloc[:min_len]

    if len(residuals_after_var) != len(ua):
        min_len = min(len(residuals_after_var), len(ua))
        residuals_after_var = residuals_after_var[:min_len]
        ua = ua.iloc[:min_len]
    
    # Add a constant term to the exogenous variables
    exog_before = add_constant(ub)
    exog_after = add_constant(ua)

    # Perform the White test on pre Nov 2008 data
    white_test_stat_before, white_test_pval_before, _, _ = het_white(residuals_before_var, exog_before)
    print("White test statistic before Nov 2008:", white_test_stat_before)
    print("White test p-value before Nov 2008:", white_test_pval_before)

    # Perform the White test on post Nov 2008 data
    white_test_stat_after, white_test_pval_after, _, _ = het_white(residuals_after_var, exog_after)
    print("White test statistic after Nov 2008:", white_test_stat_after)
    print("White test p-value after Nov 2008:", white_test_pval_after)
    print("\n")

###########################
#### Granger Causality ####
###########################

# Variables in the model
variables = results_after_nov_2008.names

# Run the Granger causality tests
for var in variables:
    remaining_vars = list(set(variables) - set([var]))
    for causing_var in remaining_vars:
        test_result = results_after_nov_2008.test_causality(var, causing_var, kind='f')
        if test_result.pvalue < 0.05:
            print(f"{causing_var} Granger-causes {var} - Test statistic: {test_result.test_statistic:.3f}, p-value: {test_result.pvalue:.3f}")
        else:
            print(f"{causing_var} does not Granger-cause {var} - Test statistic: {test_result.test_statistic:.3f}, p-value: {test_result.pvalue:.3f}")


